package main

import (
	log "github.com/sirupsen/logrus"
	"net/http"
	"os"
	"os/signal"
	"syscall"
)

var (
	serverConfig map[string]interface{}
	systemConfig map[string]interface{}
)

var (
	server     = http.Server{}
	executable string
)

var (
	r bool
	d bool
)

func init() {
	setFormatter()
	var err error
	executable, err = os.Executable()
	if err != nil {
		log.Warnf("Error while obtaining executable path, %s. Restarting will no longer work.", err)
	}
}

func main() {

	// Setup configuration stuff
	configSetup()

	// Initial config update
	setLevel()

	// Open store
	openStore()

	// Set up web
	webSetup()

	// Configure listener
	listenerSetup()

	// Register API and web handlers
	registerAPI()
	registerWebpage()

	// Signal handling
	signalChannel := make(chan os.Signal, 1)
	signal.Notify(signalChannel, syscall.SIGINT, syscall.SIGTERM, syscall.SIGHUP, os.Interrupt, os.Kill)
	go func() {
		// Cleanup on function return
		defer func() { cleanup(false) }()
		for {
			currentSignal := <-signalChannel
			switch currentSignal {
			case os.Interrupt:
				println()
				log.Info("Gracefully exiting.")
				return
			default:
				log.Info("Gracefully exiting.")
				return
			}
		}
	}()

	// Start server
	runWebServer()

	// Restart if needed
	if r {
		restart()
	}
}
